@@ -16,6 +16,8 @@ static void alpha_insert_admin(AdminInfo *bi);
 static void do_addoper(const char *source);
 static void do_adminwhois(const char *source);
 static void do_setserver(const char *source);
+static void showadmin(const char *source, int dowhat);
+
 int is_oper_override(const char *source,AdminInfo *ais);
 
 /**********************************************************************/
@@ -639,7 +641,7 @@ static void do_adminwhois(const char *source) {
             strcat(buf, "FJOIN");
        	}
         if (!*buf)
-            strcpy(buf, "None");
+            strscpy(buf, "None", 4);
 	notice(s_AdminServ, source, "Flags: %s", buf);
 
 	if (ais->flags & AI_QUIT) {
@@ -1066,24 +1068,14 @@ void os_remove_nick(const char *nick)
 	deladmin(ais);
 }
 /************************************************************/
-void sendmotd(const char *source)
-{
-    char buf[BUFSIZE];
-    NickInfo *ni;
-    AdminInfo *ais;
-    int i, avail=0, all=0;
-    *buf = 0;
-    send_cmd(server_name, "372 %s :- This list of people can retrieve"
-                " lost passwords:", source);
-    send_cmd(server_name, "372 %s :-", source);
-    ni = findnick(SERVICES_MASTER);
-    if (ni && (ni->flags & NI_IDENTIFIED)) {
-	    send_cmd(server_name, "372 %s :- Services Master \2%s\2\n", source, SERVICES_MASTER);
-    } else {
-	    send_cmd(server_name, "372 %s :- Services Master %s\n", source, SERVICES_MASTER);
-    }
-    send_cmd(server_name, "372 %s :-", source);
-    for (i = 33; i < 256; ++i) {
+static void showadmin(const char *source, int dowhat) {
+   char buf[BUFSIZE];
+   AdminInfo *ais;
+   NickInfo *ni;
+
+   int i, avail=0, all=0;
+   *buf = 0;
+   for (i = 33; i < 256; ++i) {
         for (ais = adminlists[i]; ais; ais = ais->next)
                 if ((ais->adflags & (ADF_SERVICEADMIN|ADF_SERVICEROOT)) && !(ais->flags & AI_QUIT)) {
 			all++;
@@ -1095,27 +1087,42 @@ void sendmotd(const char *source)
                 		strcat(buf, " ");
 				avail++;
 				if (strlen(buf) > 60) {
-					send_cmd(server_name, "372 %s :- %s \n", source, buf);
+					send_cmd(server_name, "372 %s :- %s", source, buf);
 					*buf=0;
 				}
 			} else {
 		                strcat(buf, ais->nick);
 		                strcat(buf, " ");
 				if (strlen(buf) > 60) {
-					send_cmd(server_name, "372 %s :- %s \n", source, buf);
+					send_cmd(server_name, "372 %s :- %s", source, buf);
 					*buf=0;
 				}
 			}
 		}
     }
-    send_cmd(server_name, "372 %s :- %s \n", source, buf);
+    send_cmd(server_name, "372 %s :- %s", source, buf);
     send_cmd(server_name, "372 %s :-", source);
     send_cmd(server_name, "372 %s :- %d of %d nicks available.",
          source, avail, all);
-
     send_cmd(server_name, "372 %s :- Note: The nicks that are in \2bold\2"
                  " are currently online", source);
+}
+/************************************************************/
+void sendmotd(const char *source)
+{
+    NickInfo *ni;
 
+    send_cmd(server_name, "372 %s :- This list of people can retrieve"
+                " lost passwords:", source);
+    send_cmd(server_name, "372 %s :-", source);
+    ni = findnick(SERVICES_MASTER);
+    if (ni && (ni->flags & NI_IDENTIFIED)) {
+	    send_cmd(server_name, "372 %s :- Services Master \2%s\2", source, SERVICES_MASTER);
+    } else {
+	    send_cmd(server_name, "372 %s :- Services Master %s", source, SERVICES_MASTER);
+    }
+    send_cmd(server_name, "372 %s :-", source);
+    showadmin(source,1) ;
 }
 
 /***********************************************/
