@@ -38,15 +38,22 @@ static void m_user(char *source, int ac, char **av)
     send_cmd (server_name, "446 %s :USERS has been disabled", source);
 }
 /*************************************************************************/
+#ifdef UNREAL
 static void m_umode2(char *source, int ac, char **av)
 {
         av[1] = sstrdup (av[0]);
-        strcpy (av[0], source);
+        strscpy (av[0], source, sizeof(source));
         do_umode (source, ac, av);
 	if (av[1]) free(av[1]);
 }
+#endif
 /*************************************************************************/
 #ifdef UNREAL
+/* av[1] ST
+   av[2] PROTO
+   av[3] CLOAK KEY?
+   av[8] Network name
+*/
 static void m_netinfo(char *source, int ac, char **av)
 {
     if (ac < 1)
@@ -54,10 +61,16 @@ static void m_netinfo(char *source, int ac, char **av)
     if (atoi (av[0]) > maxusercnt)
         maxusercnt = atoi (av[0]);
 #ifdef UNREAL32
-    send_cmd (server_name, "NETINFO %d %lu 2303 * * * * :%s",
-                maxusercnt, time(NULL), av[7]);
+    if (av[3]) {
+	if (CLOAK_KEY) free(CLOAK_KEY);
+	CLOAK_KEY = sstrdup(av[3]);
+    }
+    send_cmd (server_name, "%s %d %lu 2303 %s * * * :%s", 
+		istoken?TOK_NETINFO:MSG_NETINFO,
+                maxusercnt, time(NULL), CLOAK_KEY, av[7]);
 #else
-    send_cmd (server_name, "NETINFO %d %lu 2302 * * * * :%s",
+    send_cmd (server_name, "%s %d %lu 2302 * * * * :%s",
+		istoken?TOK_NETINFO:MSG_NETINFO,
                 maxusercnt, time(NULL), av[7]);
 #endif
 }
@@ -237,7 +250,6 @@ static void m_sethost(char *source, int ac, char **av)
 			free(u->host);
 	        u->host = sstrdup(av[0]);
 	}
-
         /* Now, update the nickinfo. */
         ni = findnick (u->nick);
 	hni = nickhost(ni);
@@ -279,7 +291,6 @@ static void m_server(char *source, int ac, char **av)
 {
     if (ac < 1)
         return;
-
     numserver++;
     check_jupes();
     check_zlines();
@@ -314,8 +325,10 @@ static void m_away(char *source, int ac, char **av)
 /*************************************************************************/
 static void m_join(char *source, int ac, char **av)
 {
+#ifndef CR
     if (ac != 1)
 	return;
+#endif
     do_join(source, ac, av);
 }
 
@@ -786,11 +799,12 @@ static void m_part(char *source, int ac, char **av)
     do_part(source, ac, av);
 }
 /*************************************************************************/
+#ifdef UNREAL
 static void m_swhois(char *source, int ac, char **av)
 {
 	send_cmd(NULL, "SWHOIS %s :", av[0]);
 }
-
+#endif
 /*************************************************************************/
 static void m_privmsg(char *source, int ac, char **av)
 {
@@ -913,22 +927,22 @@ static void m_privmsg(char *source, int ac, char **av)
 
     } else if (stricmp(av[0], s_RootServ) == 0) {
         if (is_services_root(source) && is_oper(source))
-            rootserv(source, av[1]);
+            rootserv(u, source, av[1]);
 	else {
             notice(s_RootServ, source, "Sorry, only Services Root may access RootServ.");
 	}
 
     } else if (stricmp(av[0], s_NickServ) == 0) {
-	nickserv(source, av[1]);
+	nickserv(u, source, av[1]);
 
     } else if (stricmp(av[0], s_ChanServ) == 0) {
-	chanserv(source, av[1]);
+	chanserv(u, source, av[1]);
 
     } else if (stricmp(av[0], s_MemoServ) == 0) {
-	memoserv(source, av[1]);
+	memoserv(u, source, av[1]);
 
     } else if (stricmp(av[0], s_Agent) == 0) {
-	agent(source, av[1]);
+	agent(source, av[1], u);
 
     } else if (stricmp(av[0], s_HelpServ) == 0) {
 	helpserv(s_HelpServ, source, av[1]);
@@ -949,14 +963,14 @@ static void m_privmsg(char *source, int ac, char **av)
 
 #ifdef DEFHOSTSERV
     } else if (stricmp(av[0], s_HostServ) == 0) {
-	HostServ(source, av[1]);
+	HostServ(u, source, av[1]);
 #endif
 
     } else if (stricmp(av[0], s_AdminServ) == 0) {
 	adminserv(source, av[1]);
 #ifdef WEBSERVICE
     } else if (stricmp(av[0], s_WebServ) == 0) {
-	webserv(source, av[1]);
+	webserv(u, source, av[1]);
 #endif
     } else if (stricmp(av[0], s_BotServ) == 0) {
 	sbotserv(source, av[1]);
@@ -965,7 +979,7 @@ static void m_privmsg(char *source, int ac, char **av)
         botserv(source, av[0], av[1]);
 
     } else if (*av[0] == '#') {
-	bmsgchan(source, av[0], av[1]);
+	bmsgchan(source, av[0], av[1], u);
 
     } else if (snchan && !stricmp(av[0], snchan)) {
 	send_cmd(s_OperServ, "PRIVMSG %s :<%s/%s> %s", snoopchan, source, av[0], av[1]);
@@ -1006,6 +1020,9 @@ Message messages[] = {
     { "CIDENT",     NULL },
     { "SERVERINFO", NULL },
     { "IJOIN", m_join },
+#endif
+#ifdef RAGEIRCD
+    { "SNICK",      m_nick },
 #endif
     { "380",	   NULL },
     { "401",	   NULL },
@@ -1013,51 +1030,50 @@ Message messages[] = {
     { "302",	   m_userhost },
     { "HELP",	   NULL },
     { "4",	   NULL },
-    { "AWAY",      m_away },
-    { "6",      m_away },
-    { "JOIN",      m_join },
-    { "C",      m_join },
+    { MSG_AWAY,      m_away },
+    { TOK_AWAY,      m_away },
+    { MSG_JOIN,      m_join },
+    { TOK_JOIN,      m_join },
 #ifdef SJOIN
-    { "SJOIN",     m_sjoin },
-    { "~",     m_sjoin },
+    { MSG_SJOIN,     m_sjoin },
+    { TOK_SJOIN,     m_sjoin },
 #endif
-    { "KICK",      m_kick },
-    { "H",      m_kick },
-    { "KILL",      m_kill },
-    { ".",      m_kill },
-    { "SVSKILL",   NULL },
-    { "h",   NULL },
-    { "MODE",      m_mode },
-    { "G",      m_mode },
+    { MSG_KICK,      m_kick },
+    { TOK_KICK,      m_kick },
+    { MSG_KILL,      m_kill },
+    { TOK_KILL,      m_kill },
+    { MSG_SVSKILL,   NULL },
+    { TOK_SVSKILL,   NULL },
+    { MSG_MODE,      m_mode },
+    { TOK_MODE,      m_mode },
     { "SVSMODE",   NULL },
     { "n",   NULL },
-    { "MOTD",      m_motd },
-    { "F",      m_motd },
-    { "NICK",      m_nick },
-    { "SNICK",      m_nick },
-    { "&",      m_nick },
-    { "NOTICE",    NULL },
-    { "B",    NULL },
-    { "PART",      m_part },
-    { "D",      m_part },
-    { "PASS",      NULL },
-    { "<",      NULL },
-    { "PING",      m_ping },
-    { "8",      m_ping },
-    { "PRIVMSG",   m_privmsg },
-    { "!",   m_privmsg },
-    { "QUIT",      m_quit },
-    { ",",      m_quit },
-    { "SERVER",    m_server },
-    { "'",    m_server },
-    { "SQUIT",     m_squit },
-    { "-",     m_squit },
-    { "TOPIC",     m_topic },
-    { ")",     m_topic },
+    { MSG_MOTD,      m_motd },
+    { TOK_MOTD,      m_motd },
+    { MSG_NICK,      m_nick },
+    { TOK_NICK,      m_nick },
+    { MSG_NOTICE,    NULL },
+    { TOK_NOTICE,    NULL },
+    { MSG_PART,      m_part },
+    { TOK_PART,      m_part },
+    { MSG_PASS,      NULL },
+    { TOK_PASS,      NULL },
+    { MSG_PING,      m_ping },
+    { TOK_PING,      m_ping },
+    { MSG_PRIVATE,   m_privmsg },
+    { TOK_PRIVATE,   m_privmsg },
+    { MSG_QUIT,      m_quit },
+    { TOK_QUIT,      m_quit },
+    { MSG_SERVER,    m_server },
+    { TOK_SERVER,    m_server },
+    { MSG_SQUIT,     m_squit },
+    { TOK_SQUIT,     m_squit },
+    { MSG_TOPIC,     m_topic },
+    { TOK_TOPIC,     m_topic },
     { "WALLOPS",   NULL },
     { "=",   NULL },
-    { "VERSION",   m_version },
-    { "+",   m_version },
+    { MSG_VERSION,   m_version },
+    { TOK_VERSION,   m_version },
     { "AKILL",     NULL },
     { "V",     NULL },
     { "GLOBOPS",   NULL },
@@ -1070,26 +1086,26 @@ Message messages[] = {
     { "[",     NULL },
     { "RAKILL",    NULL },
     { "Y",    NULL },
-    { "ADMIN",     m_admin },
-    { "@",     m_admin },
-    { "USERS",     m_user },
-    { "3",     m_user },
-    { "INFO",      m_info },
-    { "/",      m_info },
-    { "CREDITS",   m_credits },
-    { "AJ",   m_credits },
-    { "STATS",     m_stats },
-    { "2",     m_stats },
-    { "TIME",      m_time },
-    { ">",      m_time },
-    { "WHOIS",    NULL },
-    { "#",     NULL },
+    { MSG_ADMIN,     m_admin },
+    { TOK_ADMIN,     m_admin },
+    { MSG_USERS,     m_user },
+    { TOK_USERS,     m_user },
+    { MSG_INFO,      m_info },
+    { TOK_INFO,      m_info },
+    { MSG_CREDITS,   m_credits },
+    { TOK_CREDITS,   m_credits },
+    { MSG_STATS,     m_stats },
+    { TOK_STATS,     m_stats },
+    { MSG_TIME,      m_time },
+    { TOK_TIME,      m_time },
+    { MSG_WHOIS,    NULL },
+    { TOK_WHOIS,     NULL },
     { "SMO",       NULL },
     { "AU",       NULL },
     { "PROTOCTL",  NULL },
     { "_",  NULL },
-    { "SQLINE", m_addsqline },
-    { "c", m_addsqline },
+    { MSG_SQLINE, m_addsqline },
+    { TOK_SQLINE, m_addsqline },
     { "UNSQLINE",  NULL },
     { "d",  NULL },
     { "SVSNOOP",  NULL },
@@ -1100,22 +1116,23 @@ Message messages[] = {
     { "SWHOIS",  NULL },
     { "VCTRL",  NULL },
     { "SYNC",  NULL },
-    { "SETIDENT",  m_setident },
-    { "AD",  m_setident },
-    { "SETHOST",   m_sethost },
-    { "AA",   m_sethost },
-    { "SETNAME",   m_setname },
-    { "AE",   m_setname },
+    { MSG_SETIDENT,  m_setident },
+    { TOK_SETIDENT,  m_setident },
+    { MSG_SETHOST,   m_sethost },
+    { TOK_SETHOST,   m_sethost },
+    { MSG_SETNAME,   m_setname },
+    { TOK_SETNAME,   m_setname },
     { "SETIDENT",  NULL },
     { "AD",  NULL },
     { "CHGHOST",   NULL },
     { "AL",   NULL },
 #ifdef UNREAL
-    { "UMODE2",  m_umode2 },
-    { "|",  m_umode2 },
-    { "NETINFO",   m_netinfo },
-    { "AO",   m_netinfo },
-    { "BA",   m_swhois },
+    { MSG_UMODE2,  m_umode2 },
+    { TOK_UMODE2,  m_umode2 },
+    { MSG_NETINFO,   m_netinfo },
+    { TOK_NETINFO,   m_netinfo },
+    { MSG_SWHOIS,   m_swhois },
+    { TOK_SWHOIS,   m_swhois },
 #endif
 #ifdef ULTIMATE
     { "NETG",  NULL },
